import pytest

from chrono.agendas.models import Person, SharedCustodyAgenda

pytestmark = pytest.mark.django_db


def test_add_shared_custody_agenda(app, user, settings):
    app.authorization = ('Basic', ('john.doe', 'password'))

    params = {
        'first_guardian_name': 'John Doe',
        'first_guardian_id': 'xxx',
        'second_guardian_name': 'Jane Doe',
        'second_guardian_id': 'yyy',
    }
    resp = app.post_json('/api/shared-custody/', params=params)

    john = Person.objects.get(user_external_id='xxx', name='John Doe')
    jane = Person.objects.get(user_external_id='yyy', name='Jane Doe')

    agenda = SharedCustodyAgenda.objects.get()
    assert agenda.first_guardian == john
    assert agenda.second_guardian == jane

    assert resp.json['data'] == {
        'id': agenda.pk,
        'settings_url': 'http://testserver/manage/shared-custody/%s/settings/' % agenda.pk,
    }

    params = {
        'first_guardian_name': 'John Doe',
        'first_guardian_id': 'xxx',
        'second_guardian_name': 'Other',
        'second_guardian_id': 'zzz',
    }
    resp = app.post_json('/api/shared-custody/', params=params)
    assert resp.json['data']['id'] != agenda.pk
    assert SharedCustodyAgenda.objects.filter(first_guardian=john).count() == 2


def test_shared_custody_agenda_add_child(app, user, settings):
    father = Person.objects.create(user_external_id='father_id', name='John Doe')
    mother = Person.objects.create(user_external_id='mother_id', name='Jane Doe')
    agenda = SharedCustodyAgenda.objects.create(first_guardian=father, second_guardian=mother)

    app.authorization = ('Basic', ('john.doe', 'password'))
    params = {
        'name': 'James Doe',
        'user_external_id': 'xxx',
    }
    resp = app.post_json('/api/shared-custody/%s/add-child/' % agenda.pk, params=params)
    child = Person.objects.get(name='James Doe', user_external_id='xxx')
    assert resp.json['data']['child_id'] == child.pk

    assert agenda.children.count() == 1
    assert Person.objects.count() == 3

    # adding again throws error
    resp = app.post_json('/api/shared-custody/%s/add-child/' % agenda.pk, params=params)
    assert resp.json['err'] == 1
    assert resp.json['err_desc'] == 'This child already has one custody agenda.'

    assert agenda.children.count() == 1
    assert Person.objects.count() == 3

    params = {
        'name': 'Jack Doe',
        'user_external_id': 'yyy',
    }
    resp = app.post_json('/api/shared-custody/%s/add-child/' % agenda.pk, params=params)
    child = Person.objects.get(name='Jack Doe', user_external_id='yyy')
    assert resp.json['data']['child_id'] == child.pk

    assert agenda.children.count() == 2
    assert Person.objects.count() == 4

    other_father = Person.objects.create(user_external_id='other_father_id', name='John Doe')
    other_mother = Person.objects.create(user_external_id='other_mother_id', name='Jane Doe')
    other_agenda = SharedCustodyAgenda.objects.create(
        first_guardian=other_father, second_guardian=other_mother
    )

    resp = app.post_json('/api/shared-custody/%s/add-child/' % other_agenda.pk, params=params)
    assert resp.json['err'] == 1
    assert resp.json['err_desc'] == 'This child already has one custody agenda.'
